@@ -15,8 +15,10 @@ from xmltodict import parse as xml_parse
 from ..exceptions import OmniParsingException
 from ..types import (
     BodyOfWaterType,
+    ChlorinatorDispenserType,
     ColorLogicLightType,
     ColorLogicShow,
+    CSADType,
     FilterType,
     HeaterType,
     OmniType,
@@ -54,10 +56,9 @@ class OmniBase(BaseModel):
                     # ... then call propagate_bow_id on each of them ...
                     if device is not None:
                         device.propagate_bow_id(bow_id)
-            else:
-                # ... otherwise just call it on the single subdevice
-                if subdevice is not None:
-                    subdevice.propagate_bow_id(bow_id)
+            # ... otherwise just call it on the single subdevice
+            elif subdevice is not None:
+                subdevice.propagate_bow_id(bow_id)
 
 
 class MSPSystem(BaseModel):
@@ -134,8 +135,42 @@ class MSPVirtualHeater(OmniBase):
 
         # The heater equipment are nested down inside a list of "Operations", which also includes non Heater-Equipment items.  We need to
         # first filter down to just the heater equipment items, then populate our self.heater_equipment with parsed versions of those items.
-        heater_equip_data = [op for op in data.get("Operation", {}) if OmniType.HEATER_EQUIP in op][0]
-        self.heater_equipment = [MSPHeaterEquip.parse_obj(equip) for equip in heater_equip_data[OmniType.HEATER_EQUIP]]
+        heater_equip_data = [op[OmniType.HEATER_EQUIP] for op in data.get("Operation", {}) if OmniType.HEATER_EQUIP in op]
+        self.heater_equipment = [MSPHeaterEquip.parse_obj(equip) for equip in heater_equip_data]
+
+
+class MSPChlorinatorEquip(OmniBase):
+    omni_type: OmniType = OmniType.CHLORINATOR_EQUIP
+    enabled: Literal["yes", "no"] = Field(alias="Enabled")
+
+
+class MSPChlorinator(OmniBase):
+    _sub_devices = {"chlorinator_equipment"}
+
+    omni_type: OmniType = OmniType.CHLORINATOR
+    enabled: Literal["yes", "no"] = Field(alias="Enabled")
+    timed_percent: int = Field(alias="Timed-Percent")
+    superchlor_timeout: int = Field(alias="SuperChlor-Timeout")
+    dispenser_type: ChlorinatorDispenserType | str = Field(alias="Dispenser-Type")
+    chlorinator_equipment: list[MSPChlorinatorEquip] | None
+
+    def __init__(self, **data: Any) -> None:
+        super().__init__(**data)
+
+        # The heater equipment are nested down inside a list of "Operations", which also includes non Heater-Equipment items.  We need to
+        # first filter down to just the heater equipment items, then populate our self.heater_equipment with parsed versions of those items.
+        chlorinator_equip_data = [op for op in data.get("Operation", {}) if OmniType.CHLORINATOR_EQUIP in op][0]
+        self.chlorinator_equipment = [MSPChlorinatorEquip.parse_obj(equip) for equip in chlorinator_equip_data[OmniType.CHLORINATOR_EQUIP]]
+
+
+class MSPCSAD(OmniBase):
+    omni_type: OmniType = OmniType.CSAD
+    enabled: Literal["yes", "no"] = Field(alias="Enabled")
+    type: CSADType | str = Field(alias="Type")
+    target_value: float = Field(alias="TargetValue")
+    calibration_value: float = Field(alias="CalibrationValue")
+    ph_low_alarm_value: float = Field(alias="PHLowAlarmLevel")
+    ph_high_alarm_value: float = Field(alias="PHHighAlarmLevel")
 
 
 class MSPColorLogicLight(OmniBase):
@@ -150,16 +185,19 @@ class MSPColorLogicLight(OmniBase):
 
 
 class MSPBoW(OmniBase):
-    _sub_devices = {"filter", "relay", "heater", "sensor", "colorlogic_light", "pump"}
+    _sub_devices = {"filter", "relay", "heater", "sensor", "colorlogic_light", "pump", "chlorinator", "csad"}
 
     omni_type: OmniType = OmniType.BOW
     type: BodyOfWaterType | str = Field(alias="Type")
+    supports_spillover: Literal["yes", "no"] = Field(alias="Supports-Spillover")
     filter: list[MSPFilter] | None = Field(alias="Filter")
     relay: list[MSPRelay] | None = Field(alias="Relay")
     heater: MSPVirtualHeater | None = Field(alias="Heater")
     sensor: list[MSPSensor] | None = Field(alias="Sensor")
     colorlogic_light: list[MSPColorLogicLight] | None = Field(alias="ColorLogic-Light")
     pump: list[MSPPump] | None = Field(alias="Pump")
+    chlorinator: MSPChlorinator | None = Field(alias="Chlorinator")
+    csad: list[MSPCSAD] | None = Field(alias="CSAD")
 
     # We override the __init__ here so that we can trigger the propagation of the bow_id down to all of it's sub devices after the bow
     # itself is initialized
@@ -205,12 +243,13 @@ class MSPConfig(BaseModel):
             # everything that *could* be a list into a list to make the parsing more consistent.
             force_list=(
                 OmniType.BOW_MSP,
-                OmniType.CHLORINATOR,
+                OmniType.CHLORINATOR_EQUIP,
+                OmniType.CSAD,
                 OmniType.CL_LIGHT,
                 OmniType.FAVORITES,
                 OmniType.FILTER,
                 OmniType.GROUPS,
-                OmniType.HEATER_EQUIP,
+                OmniType.PUMP,
                 OmniType.RELAY,
                 OmniType.SENSOR,
                 OmniType.SCHE,
